#include "core/application.h"

#include "core/config.h"

#include "core/notificationsdbus.h"
#include "core/ucpufreq.h"
#include "core/upower.h"
#include "core/dbuswatcher.h"
#include "gui/mainwindow.h"
#include "gui/systemtray.h"

#include <QSettings>

using namespace core;

Application::Application(int &argv, char *argc[]) : QApplication(argv, argc) {
    setApplicationName("BatterySystemTray");
    setApplicationVersion("1.0.0.rc2");

    setOrganizationName("PyrOTools");
    setOrganizationDomain("pyrotools.org");

    setQuitOnLastWindowClosed(false);
}

Application::~Application() {
    delete m_mainwindow;
}

void Application::init() {
    m_watcher = new DBusWatcher(this);
    m_cpufreq = new UCpufreq(this);
    m_upower = new UPower(this);
    m_notification = 0;
    m_mainwindow = new gui::MainWindow;
    m_systemtray = new gui::SystemTray(this);

    connect(m_systemtray, SIGNAL(showClicked()), m_mainwindow, SLOT(display()));
    connect(m_systemtray, SIGNAL(minimizeClicked()), m_mainwindow, SLOT(hide()));
    connect(m_systemtray, SIGNAL(maximizeClicked()), m_mainwindow, SLOT(show()));

    connect(m_watcher, SIGNAL(notificationsAvailable()), this, SLOT(reconnectNotification()));

    reconnectNotification();
}

void Application::reconnectNotification() {
    if(m_notification)
        delete m_notification;
    m_notification = new DBusNotification(DBUS_INTERFACE_NOTIFICATIONS, DBUS_OBJECT_NOTIFICATIONS, QDBusConnection::sessionBus(), this);
}
