#include "dbuswatcher.h"

#include "QDBusConnection"

using namespace core;

DBusWatcher::DBusWatcher(QObject *parent) : QObject(parent) {
    m_system.setConnection(QDBusConnection::systemBus());
    m_session.setConnection(QDBusConnection::sessionBus());
    m_system.addWatchedService(upowerInterface());
    m_system.addWatchedService(consoleKitInterface());
    m_system.addWatchedService(ucpufreqInterface());
    m_session.addWatchedService(notificationInterface());

    m_system.setWatchMode(QDBusServiceWatcher::WatchForRegistration | QDBusServiceWatcher::WatchForUnregistration);
    m_session.setWatchMode(QDBusServiceWatcher::WatchForRegistration | QDBusServiceWatcher::WatchForUnregistration);

    connect(&m_system, SIGNAL(serviceRegistered(QString)), this, SLOT(serviceRegistered(QString)));
    connect(&m_system, SIGNAL(serviceUnregistered(QString)), this, SLOT(serviceUnregistered(QString)));
    connect(&m_session, SIGNAL(serviceRegistered(QString)), this, SLOT(serviceRegistered(QString)));
    connect(&m_session, SIGNAL(serviceUnregistered(QString)), this, SLOT(serviceUnregistered(QString)));
}

void DBusWatcher::serviceRegistered(QString service) {
    Service s = serviceType(service);
    if(s != Unknown) {
        emit serviceAvailable(s);
        switch(s) {
            case UPower:
                emit upowerAvailable();
                break;
            case ConsoleKit:
                emit consoleKitAvailable();
                break;
            case UCpufreq:
                emit ucpufreqAvailable();
                break;
            case Notification:
                emit notificationsAvailable();
                break;
            default:
                break;
        }
    }
}

void DBusWatcher::serviceUnregistered(QString service) {
    Service s = serviceType(service);
    if(s != Unknown) {
        emit serviceUnavailable(s);
        switch(s) {
            case UPower:
                emit upowerUnavailable();
                break;
            case ConsoleKit:
                emit consoleKitUnavailable();
                break;
            case UCpufreq:
                emit ucpufreqUnavailable();
                break;
            case Notification:
                emit notificationsUnavailable();
                break;
            default:
                break;
        }
    }
}

DBusWatcher::Service DBusWatcher::serviceType(const QString &service) const {
    if(service.compare(upowerInterface()) == 0) {
        return UPower;
    } else if(service.compare(consoleKitInterface()) == 0) {
        return ConsoleKit;
    } else if(service.compare(ucpufreqInterface()) == 0) {
        return UCpufreq;
    } else if(service.compare(notificationInterface()) == 0) {
        return Notification;
    }
    return Unknown;
}
