#ifndef DBUSNOTIFICATION_H
#define DBUSNOTIFICATION_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

#include "core/application.h"

namespace core {

class DBusNotification : public QDBusAbstractInterface {

public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.Notification"; }

public:
    DBusNotification(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);
    ~DBusNotification();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Notify(QString app_name, uint id, QString icon, QString summary, QString body, QStringList actions, QVariantMap hints, int timeout)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(app_name)
                     << QVariant::fromValue(id)
                     << QVariant::fromValue(icon)
                     << QVariant::fromValue(summary)
                     << QVariant::fromValue(body)
                     << QVariant::fromValue(actions)
                     << QVariant::fromValue(hints)
                     << QVariant::fromValue(timeout);
        return asyncCallWithArgumentList(QLatin1String("Notify"), argumentList);
    }

    inline QDBusPendingReply<> Notify(QString summary, QString body, QString icon, int timeout = 2000) {
        return Notify(qApp->applicationName(), static_cast<unsigned int>(0), icon, summary, body, QStringList(), QVariantMap(), timeout);
    }
};

}

#endif // DBUSNOTIFICATION_H
