#include "core/ucpufreq.h"

#include "core/ucpufreqdbus.h"
#include "core/cpudbus.h"

#include "core/config.h"

#include "core/application.h"
#include "core/dbuswatcher.h"

using namespace core;

UCpufreq::UCpufreq(QObject *parent) : QObject(parent),
    m_interface(0) {
    connect(app->dbusWatcher(), SIGNAL(ucpufreqAvailable()), this, SLOT(reconnect()));
    connect(app->dbusWatcher(), SIGNAL(ucpufreqUnavailable()), this, SIGNAL(disconnected()));
    connect(app->dbusWatcher(), SIGNAL(ucpufreqUnavailable()), this, SLOT(clearCPUs()));
    reconnect();
}

UCpufreq::~UCpufreq() {

}

void UCpufreq::reconnect() {
    if(m_interface)
        delete m_interface;
    m_interface = new UCpufreqDBus(DBUS_INTERFACE_UCPUFREQ, DBUS_OBJECT_UCPUFREQ, QDBusConnection::systemBus(), this);
    if(m_interface->isValid()) {
        emit connected();
        discoverCPUs();
    }
}

bool UCpufreq::isConnected() const {
    return m_interface->isValid();
}

void UCpufreq::discoverCPUs() {
    QDBusPendingReply<QList<QDBusObjectPath> > reply = m_interface->EnumerateCPUs();
    QDBusPendingCallWatcher *w = new QDBusPendingCallWatcher(reply);
    connect(w, SIGNAL(finished(QDBusPendingCallWatcher*)), this, SLOT(discoveringFinished(QDBusPendingCallWatcher*)));
}

void UCpufreq::discoveringFinished(QDBusPendingCallWatcher *w) {
    QDBusPendingReply<QList<QDBusObjectPath> > reply = *w;
    if(!reply.isError() && reply.isFinished() && reply.isValid()) {
        QList<QDBusObjectPath> list = reply.value();
        foreach(QDBusObjectPath path, list) {
            addCpu(path);
        }
    }
    w->deleteLater();
}

void UCpufreq::addCpu(QDBusObjectPath path) {
    CpuDBus *cpu = new CpuDBus(DBUS_INTERFACE_UCPUFREQ, path.path(), m_interface->connection(), this);
    m_cpus.insert(path.path(), cpu);
    emit cpuAdded(cpu);
}

core::CpuDBus* UCpufreq::cpu(const QString &path) {
    if(m_cpus.contains(path)) {
        return m_cpus[path];
    }
    return 0;
}

void UCpufreq::clearCPUs() {
    foreach(CpuDBus *c, m_cpus) {
        c->deleteLater();
    }
    m_cpus.clear();
}

void UCpufreq::setGovernor(const QString &governor) {
    foreach(CpuDBus *c, m_cpus) {
        c->setGovernor(governor);
    }
}

QString UCpufreq::governor() const {
    if(m_cpus.begin().value())
        return m_cpus.begin().value()->governor();
    return QString();
}

void UCpufreq::setFrequency(qulonglong frequency) {
    foreach(CpuDBus *c, m_cpus) {
        c->setFrequency(frequency);
    }
}

qulonglong UCpufreq::frequency() const {
    if(m_cpus.begin().value())
        return m_cpus.begin().value()->frequency();
    return 0;
}

void UCpufreq::clear() {
    m_interface->Clear();
}

void UCpufreq::save() {
    m_interface->Save();
}

void UCpufreq::load() {
    m_interface->Load();
}
