#ifndef SYSTEMTRAY_H
#define SYSTEMTRAY_H

#include <QSystemTrayIcon>

class QAction;
class QActionGroup;

namespace core {
    class Device;
}

namespace gui {

class SystemTray : public QSystemTrayIcon {
    Q_OBJECT

    public:
        SystemTray(QObject *parent = 0);
        ~SystemTray();

        bool event(QEvent *event);

    private:
        void createActions();
        void createIcons();

        QAction *m_s2ram;
        QAction *m_s2disk;
        QAction *m_reboot;
        QAction *m_halt;
        QAction *m_minimizeAction;
        QAction *m_restoreAction;
        QAction *m_quitAction;
        QMenu *m_menu;

        QMenu *m_ucpufreq_menu;
        QMenu *m_frequencies_menu;
        QMenu *m_governors_menu;

        QActionGroup *m_governorsActionGroup;
        QActionGroup *m_frequenciesActionGroup;

        QList<QAction*> m_governors;

    private Q_SLOTS:
        void iconActivated(QSystemTrayIcon::ActivationReason);
        void setMainDevice(core::Device*, core::Device*);
        void connectDevice(core::Device*);
        void mainDeviceChanged();

        void deviceOnline();
        void deviceOffline();
        void deviceLow();
        void deviceFull();
        void ucpufreqConnected();
        void ucpufreqDisconnected();

        void updateGovernors();
        void updateFrequencies();

        void changeGovernor(QAction*);
        void changeFrequency(QAction*);

private:
        static QString frequencyToString(qulonglong f);

        QStringList m_governorsList;

Q_SIGNALS:
        void minimizeClicked();
        void maximizeClicked();
        void showClicked();

};

}

#endif // SYSTEMTRAY_H
